const fs = require('fs');
const path = require('path');
const ncp = require('ncp').ncp;

var files = module.exports = {
	getCurrentDirectoryBase : function() {
		return path.basename(process.cwd());
	},

	directoryExists : function(filePath) {
		try {
			return fs.statSync(filePath).isDirectory();
		} catch (err) {
			return false;
		}
	},

	fileExists : function(filePath) {
		try {
			return fs.existsSync(filePath);
		} catch (err) {
			return false;
		}
	},

	getFileInCurrentContext : function(filePath) {
		return path.resolve(__dirname, filePath);
	},

	createDir : function(name) {
		var dir = "./" + name;
		if (!fs.existsSync(dir)) fs.mkdirSync(dir);
	},

	appendFile : function(file, text) {
		fs.appendFile(file, text, function(err) {
			if(err) return console.log(err);
		}); 
	},

	removeDir : function(name) {
		var dir = "./" + name;
		if (fs.existsSync(dir)) {
			fs.readdirSync(dir).forEach(function(file, index) {
				var curPath = dir + "/" + file;
				if (fs.lstatSync(curPath).isDirectory()) files.removeDir(curPath);
				else fs.unlinkSync(curPath);
			});
			fs.rmdirSync(dir);
		}
	},

	removeFile : function(name) {
		var pathName = "./" + name;
		if (fs.existsSync(pathName)) fs.unlinkSync(pathName);
	},

	remove : function(name) {
		var pathName = "./" + name;
		if (fs.existsSync(pathName) && fs.lstatSync(pathName).isDirectory()) files.removeDir(pathName);
		else files.removeFile(pathName);
	},

	readFile : function(filePath, callback) {
		if (fs.existsSync(filePath)) fs.readFile(filePath, 'utf8', callback);
	},

	readFileSync : function(filePath) {
		if (fs.existsSync(filePath)) return fs.readFileSync(filePath, 'utf8');
	},

	readDir : function(name, callback) {
		var dir = "./" + name;
		if (fs.existsSync(dir)) fs.readdir(dir, callback);
	},

	getChildrenDir : function(dir) {
		var dirlist = [];
		fs.readdirSync(dir).forEach(file => {
			if (fs.statSync(path.join(dir, file)).isDirectory()) dirlist = dirlist.concat(path.join(dir, file));
		});
		return dirlist;
	},

	getAllFiles : function(dir, filelist = []) {
		fs.readdirSync(dir).forEach(file => {
			filelist = fs.statSync(path.join(dir, file)).isDirectory() ? files.getAllFiles(path.join(dir, file), filelist) : filelist.concat(path.join(dir, file));
		});
		return filelist;
	},

	writeFile : function(filePath, message, callback) {
		if (fs.existsSync(filePath)) fs.writeFile(filePath, message, callback);
	},

	simpleCopy(oldFile, newFile) {
		return new Promise(function (resolve) {
			if (fs.existsSync(oldFile)) {
				ncp(oldFile, newFile, function (err) {
					if (err) return console.error(err);
					resolve();
				});
			} else console.log('Le fichier ' + oldFile + " n'existe pas.")
		});
		
	},

	copy : function(dirFilesToCopy, counter, replace) {
		if (replace) files.remove(dirFilesToCopy[counter].dest);
		return new Promise(function (resolve) {
			return ncp(dirFilesToCopy[counter].src, dirFilesToCopy[counter].dest, function (err) {
				if (err) return console.error(err);
				counter++;
				if (counter >= dirFilesToCopy.length) resolve();
				else files.copy(dirFilesToCopy, counter, replace).then(() => resolve());
			});
		});
	}
};