const exec = require('child_process').exec;
const files = require('../lib/files');
const svn = require('../lib/svn');
const config = require('../lib/config');

var utils = module.exports = {
	runCommand : function(cmd, callback, cwd, isOutput) {
		var folderExec = cwd || null;
		var command = exec(cmd, {cwd: folderExec}, callback);
		command.stdout.on('data', function (data) {
			if (isOutput) console.log(data);
		});
	},

	updateSvn : function(projectName, svnToCopy, counter) {
		return new Promise(function (resolve) {
			return svn.copy(projectName, svnToCopy[counter].src, svnToCopy[counter].dest).then(() => {
				counter++;
				if (counter >= svnToCopy.length) resolve();
				else utils.updateSvn(projectName, svnToCopy, counter).then(() => resolve());
			});
		});
	},

	commit : function(projectName) {
		return new Promise(function (resolve) {
			files.readFile(files.getFileInCurrentContext('svn.ignore'), (err, data) => {
				var excludes = data.toString().split("\r\n");
				files.readDir(projectName, (err, results) => {
					results.forEach(result => {
						if (excludes.indexOf(result) == -1) svn.add(projectName+"/"+result);
					});
					return svn.commit(projectName).then(() => resolve());
				})
			})
		});		
	},

	updateProject : function(projectName) {
		return new Promise(function (resolve) {
			var dirFiles = config.updates;
			files.createDir(projectName+"/tmp");
			svn.add(projectName+"/tmp").then(() => {
				var svnToCopy = [];
				var updateOriginals = [];
				var updateWc = [];
				for (var i = 0; i < dirFiles.length; i++) {
					var dirFile = dirFiles[i];
					svnToCopy.push({src:dirFile.svn,dest:"tmp/"+dirFile.wc});
					updateOriginals.push({src:projectName+"/tmp/"+dirFile.wc,dest:projectName+"/original/"+dirFile.wc});
					updateWc.push({src:projectName+"/tmp/"+dirFile.wc,dest:projectName+"/"+dirFile.wc});
				}
				utils.updateSvn(projectName, svnToCopy, 0).then(() => {
					files.createDir(projectName+"/original");
					// files.remove(projectName+"/original/*");
					files.copy(updateOriginals, 0, true).then(() => {
						files.copy(updateWc, 0, true).then(() => {
							return svn.delete(projectName+"/tmp").then(() => resolve());
						});
					});
				});
			});
		});
	},

	freezePackageJson : function(projectName) {
		return new Promise(function (resolve) {
			var packageFile = projectName+"/package.json";
			var packageLockFile = projectName+"/package-lock.json";
			var jsonString = files.readFileSync(packageFile).replace(/\^/g,"");
			var jsonPackage = JSON.parse(jsonString);
			jsonPackage.cordovaPlugins = [];
			files.writeFile(packageFile, JSON.stringify(jsonPackage, null, 4), () => {
				console.log("Fichier " + packageFile + " modifié.");
				files.remove(packageLockFile)
				console.log("Fichier " + packageLockFile + " supprimé.")
				resolve(jsonPackage);
			});
		});
	},

	addPlugins : function(projectName, plugins, counter) {
		return new Promise(function (resolve) {
			var plugin = plugins[counter];
			var variable = plugin.indexOf('barcode') > 0 ? ' --variable CAMERA_USAGE_DESCRIPTION="Scan QR-Codes"' : '';
			return utils.runCommand('ionic cordova plugin add ' + plugin + variable + ' --force', () => {
				console.log('Plugin ' + plugin + ' added');
				counter++;
				if (counter >= plugins.length) resolve();
				else utils.addPlugins(projectName, plugins, counter).then(() => resolve());					
			}, projectName);
		});
	},
	

	log : function(text, filePath, onlyInFile) {	
		if (!onlyInFile) console.log(text);
		files.appendFile(filePath, text + "\n");
	}
};